require "json"
require "logger"

class DeployConfig
  def initialize(config_file)
    read_config config_file
    verify_config
  end

  def each_pair(&block)
    raise "Should call with a block" unless block_given?
    @config.each_pair do |key, value|
      block.call(key, value)
    end
  end   

  def to_s
    @config.collect {|pair| "#{pair.first} => #{pair.last}"}.join("\n")
  end

  def get_binding
    each_pair do |key, value|
      instance_variable_set "@#{key}", value
    end     
    binding
  end

  def [](key)
    raise "Unknown configuration item #{key}" unless @config.has_key? key
    @config[key]
  end

  def []=(key, value)
    @config[key] = value
  end

  private

  def verify_config
    assert_no_empty_config_items
  end

  def assert_no_empty_config_items
    nullable_items = allow_null_items.split(",")
    empty_config_items = @config.select { |key| @config[key].empty? && !nullable_items.include?(key)}
    unless empty_config_items.empty?
      Process.abort "Configuration items should not be empty: #{empty_config_items.keys.join(",")}"
    end
  end

  def allow_null_items
    @config["_nullable_items"] || ""
  end

  def log(message)
    @logger ||= Logger.new(STDOUT)
    @logger.info message
  end

  def read_config(config_file)
    @config = parse_config config_file
  end

  def parse_config(config_file)
    content = IO.readlines(config_file).join("")
    content = "{}" if content.empty?
    JSON.parse(content)     
  end

  def method_missing(method, *args, &block)
    key = method.to_s
    raise "Unknown configuration item: #{method}" unless @config.has_key?(key)
    @config[key]
  end
end
