require "json"
require "erb"
require "fileutils"

module MoodeAutoDeployKit
  module ScriptProvider
    def create_script root_dir
      puts "Creating scripts to #{root_dir}"
      app_root_dir = File.join(root_dir, 'app')
      server_root_dir = File.join(root_dir, 'server')

      copy_templates_to_working_dir
      render_templates_according_to_config
      copy_generated_scripts_to_target_dir root_dir
      modify_server_gemfile server_root_dir
    end

    private

    def modify_server_gemfile server_root_dir
      extra_server_gems = IO.readlines($fs.server_gemfile).join
      target_server_gem_file = File.join(server_root_dir, 'Gemfile')
      gemfile_content  = IO.readlines(target_server_gem_file).join
      unless gemfile_content.include? extra_server_gems
        gemfile_content += extra_server_gems
      end
      File.open(target_server_gem_file, 'w') { |file| file.write(gemfile_content) }
    end

    def copy_templates_to_working_dir
      working_dir = $fs.deploy_script_working_dir
      reset_dir working_dir
      FileUtils.cp_r "#{$fs.deploy_script_template_dir}/.", working_dir
    end

    def render_templates_according_to_config
      templates = [] 
      get_template_list($fs.deploy_script_working_dir, templates)
      templates.each { |template| render_template(template, $config) }
    end

    def copy_generated_scripts_to_target_dir root_dir
      FileUtils.cp_r "#{$fs.deploy_script_working_dir}/.", root_dir
      FileUtils.rm_rf $fs.working
    end

    def render_template(template_file, config)
      puts "Rendering template #{template_file}"
      template = ERB.new(IO.readlines(template_file).join)      
      script_content = template.result($config.get_binding)
      File.open(template_file, 'w') { |file| file.write(script_content) }
    end

    def get_template_list(dir, templates)
      Dir.foreach(dir) do |entry|
        absolute_path = File.join(dir, entry)
        next if (entry == "." || entry == "..")
        templates << absolute_path if File.file?(absolute_path)
        get_template_list(absolute_path, templates) if File.directory?(absolute_path)
      end
    end    
  end
end
